# encoding: utf-8
import datetime
from south.db import db
from south.v2 import DataMigration
from django.db import models

class Migration(DataMigration):

    def forwards(self, orm):
        "Write your forwards methods here."
        for pagewidget in orm['samklang_pages.PageWidget'].objects.all():
            w = orm.Widget()
            w.page = orm.Menu.objects.get(url=pagewidget.page.url)
            w.options = pagewidget.options
            w.position = pagewidget.position
            w.into_id = "sidebar_top"
            w.widget_name = pagewidget.widget_name
            w.save()



    def backwards(self, orm):
        "Write your backwards methods here."

        for widget in orm.Widget.objects.all():
            widget.delete()


    models = {
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'ordering': "('content_type__app_label', 'content_type__model', 'codename')", 'unique_together': "(('content_type', 'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'samklang_menu.menu': {
            'Meta': {'ordering': "['name']", 'object_name': 'Menu', 'db_table': "'samklang_menu'"},
            'group': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.Group']", 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'level': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'lft': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '20'}),
            'parent': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'children'", 'null': 'True', 'to': "orm['samklang_menu.Menu']"}),
            'rght': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'tree_id': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'updated_date': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'url': ('django.db.models.fields.CharField', [], {'max_length': '100', 'db_index': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']"})
        },
        'samklang_menu.widget': {
            'Meta': {'ordering': "('position', 'widget_name')", 'object_name': 'Widget', 'db_table': "'samklang_widget'"},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'into_id': ('django.db.models.fields.CharField', [], {'max_length': '20'}),
            'options': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'page': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'widgets'", 'to': "orm['samklang_menu.Menu']"}),
            'position': ('django.db.models.fields.IntegerField', [], {}),
            'widget_name': ('django.db.models.fields.CharField', [], {'max_length': '30'})
        },
        'samklang_pages.page': {
            'Meta': {'ordering': "('url',)", 'object_name': 'Page', 'db_table': "'samklang_page'"},
            'admingroup': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'administers_pages'", 'to': "orm['auth.Group']"}),
            'content': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'content_html': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'group': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'pages'", 'null': 'True', 'to': "orm['auth.Group']"}) ,
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'site': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['sites.Site']"}),
            'url': ('django.db.models.fields.CharField', [], {'max_length': '100', 'db_index': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']"})
        },
        'samklang_pages.pagewidget': {
            'Meta': {'ordering': "('position', 'widget_name')", 'object_name': 'PageWidget', 'db_table': "'samklang_pagewidget'"},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'options': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'page': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'widgets'", 'to': "orm['samklang_pages.Page']"}),
            'position': ('django.db.models.fields.IntegerField', [], {}),
            'widget_name': ('django.db.models.fields.CharField', [], {'max_length': '30'})
        },
        'sites.site': {
            'Meta': {'ordering': "('domain',)", 'object_name': 'Site', 'db_table': "'django_site'"},
            'domain': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        }
    }

    complete_apps = ['samklang_menu']




class LazyActive(object):
    def __get__(self, request, obj_type=None):
        if not hasattr(request, '_cached_active'):
            from samklang_menu.models import Menu

            request._cached_active = Menu.find_active(request.site, request.path_info)
        return request._cached_active

class ActivePageMiddleware(object):
    def process_request(self, request):
        request.__class__.active = LazyActive()
        return None




from django.db import models
from django.contrib.auth.models import User, Group
from django.utils.translation import ugettext_lazy as _
from django.utils import simplejson
from mptt.models import MPTTModel

class Menu(MPTTModel):
    """MPTT tree menu"""
    name = models.CharField(max_length=20)
    url = models.CharField(max_length=100, db_index=True, help_text=_("Internal url starting with / or external url starting with http"))
    user = models.ForeignKey(User)
    group = models.ForeignKey(Group, null=True, blank=True)
    updated_date = models.DateTimeField(auto_now=True)
    parent = models.ForeignKey('self', null=True, blank=True, related_name='children')

    class Meta:
        ordering = ["name"]
        verbose_name, verbose_name_plural = "menu", "menus"
        db_table = 'samklang_menu'

    class MPTTMeta:
        pass

    def __unicode__(self):
        return self.name

    @models.permalink
    def get_absolute_url(self):
        return ('Menu', [self.id])

    @classmethod
    def find_active(cls, site, path):
        """
        Find active link by stripping off one part of the url at a time.
        Important to not match "anything" to the site root.
        """

        active_link = None
        while not active_link:
            # special case empty input, which probably means root
            if path == "":
                path = "/"

            try:
                active_link = cls.objects.get(tree_id=site.id, url=path)
            except cls.DoesNotExist:
                if path.endswith("/"):
                    path = path.rsplit("/", 2)[0] + "/"
                else:
                    path = path.rsplit("/", 1)[0] + "/"

            # break out before "anything" is matched to root page
            if path == "/":
                break
        return active_link

    def make_menu(self, user):
        """
        Run this on a parent to get out all children.
        """

        menu = []
        for child in self.get_children():
            if not child.group or child.group in user.groups.all():
                menu.append(child)
        return menu

    def submenu(self, user):
        """
        Will actually get submenu of the first parent it can show
        """

        active_link = self
        menu = active_link.make_menu(user)
        while not menu and active_link.parent:
            active_link = active_link.parent
            menu = active_link.make_menu(user)
        return menu


class Widget(models.Model):
    page = models.ForeignKey(Menu, verbose_name=_('Linked page'), related_name='widgets')
    widget_name = models.CharField(max_length=30)
    into_id = models.CharField(max_length=20)
    position = models.IntegerField(verbose_name=_('position'))
    options = models.TextField(blank=True, verbose_name=_('options'))

    class Meta:
        verbose_name = _('Page widget')
        verbose_name_plural = _('Page widgets')
        ordering = ('position', 'widget_name')
	db_table = 'samklang_widget'

    def __unicode__(self):
        return u"%s: %s" % (self.widget_name, self.page.name)

    def widget(self):
        app, widget_name = self.widget_name.rsplit(".", 1)
        imp = __import__(app + ".widgets", globals(), locals(), [widget_name])
        widget_class = getattr(imp, widget_name)
        try:
            options = simplejson.loads(self.options)
        except simplejson.JSONDecodeError:
            options = {}
        widget = widget_class(options)
        return widget



# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Adding model 'Widget'
        db.create_table('samklang_widget', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('page', self.gf('django.db.models.fields.related.ForeignKey')(related_name='widgets', to=orm['samklang_menu.Menu'])),
            ('widget_name', self.gf('django.db.models.fields.CharField')(max_length=30)),
            ('into_id', self.gf('django.db.models.fields.CharField')(max_length=20)),
            ('position', self.gf('django.db.models.fields.IntegerField')()),
            ('options', self.gf('django.db.models.fields.TextField')(blank=True)),
        ))
        db.send_create_signal('samklang_menu', ['Widget'])


    def backwards(self, orm):
        
        # Deleting model 'Widget'
        db.delete_table('samklang_widget')


    models = {
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'ordering': "('content_type__app_label', 'content_type__model', 'codename')", 'unique_together': "(('content_type', 'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'samklang_menu.menu': {
            'Meta': {'ordering': "['name']", 'object_name': 'Menu', 'db_table': "'samklang_menu'"},
            'group': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.Group']", 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'level': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'lft': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '20'}),
            'parent': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'children'", 'null': 'True', 'to': "orm['samklang_menu.Menu']"}),
            'rght': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'tree_id': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'updated_date': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'url': ('django.db.models.fields.CharField', [], {'max_length': '100', 'db_index': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']"})
        },
        'samklang_menu.widget': {
            'Meta': {'ordering': "('position', 'widget_name')", 'object_name': 'Widget', 'db_table': "'samklang_widget'"},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'into_id': ('django.db.models.fields.CharField', [], {'max_length': '20'}),
            'options': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'page': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'widgets'", 'to': "orm['samklang_menu.Menu']"}),
            'position': ('django.db.models.fields.IntegerField', [], {}),
            'widget_name': ('django.db.models.fields.CharField', [], {'max_length': '30'})
        }
    }

    complete_apps = ['samklang_menu']



from django.conf.urls.defaults import *
from samklang_menu.views import list_menu, new_menuitem, delete_menuitem

urlpatterns = patterns('',
    (r'^$', list_menu, {}, 'menu-list'),
    (r'^new/$', new_menuitem, {}, 'menu-new'),
    (r'^delete/$', delete_menuitem, {}, 'menu-delete'),
)



"""
This file demonstrates two different styles of tests (one doctest and one
unittest). These will both pass when you run "manage.py test".

Replace these with more appropriate tests for your application.
"""

from django.test import TestCase

from django.contrib.auth.models import User
from django.contrib.sites.models import Site

from samklang_menu.models import Menu

class FindActiveTest(TestCase):

    def test_find_main(self):
        """
        Find main page even if main page is not in menu. Also checks empty
        search.
        """

        s = Site.objects.get(pk=1) # we always have a site: from django
        m = Menu(name="parent", url="/sitemap/", tree_id=s.id, user_id=1)
        m.save()
        m2 = Menu(name="main", url="/", tree_id=s.id, parent=m, user_id=1)
        m2.save()
        self.assertEqual(m2, Menu.find_active(s, "/"))
        self.assertEqual(m2, Menu.find_active(s, ""))

    def test_find_forum_new(self):
        """
        /forum/new/ does not exist, but /forum/ does. Checks for /forum/new
        without trailing slash as well.
        """

        s = Site.objects.get(pk=1) # we always have a site: from django
        m = Menu(name="parent", url="/sitemap/", tree_id=s.id, user_id=1)
        m.save()
        m2 = Menu(name="main", url="/", tree_id=s.id, parent=m, user_id=1)
        m2.save()
        m3 = Menu(name="forum", url="/forum/", tree_id=s.id, parent=m, user_id=1)
        m3.save()
        self.assertEqual(m3, Menu.find_active(s, "/forum/new/"))
        self.assertEqual(m3, Menu.find_active(s, "/forum/new"))

class MakeMenuTest(TestCase):

    def test_submenu(self):
        """
        Test making a simple menu.
        """

        s = Site.objects.get(pk=1) # we always have a site: from django
        u = User()
        m = Menu(name="parent", url="/sitemap/", tree_id=s.id, user_id=1)
        m.save()
        m2 = Menu(name="main", url="/", tree_id=s.id, parent=m, user_id=1)
        m2.save()
        m3 = Menu(name="forum", url="/forum/", tree_id=s.id, parent=m, user_id=1)
        m3.save()
        self.assertEqual([m2, m3], m3.submenu(u))




from django.http import HttpResponseRedirect, HttpResponse, Http404
from django.template import RequestContext
from django.core.urlresolvers import reverse
from django.shortcuts import render_to_response
from django.utils import simplejson
from django.db import connection, transaction
from django.conf import settings

from samklang_menu.models import Menu
from samklang_menu.forms import MenuForm

def list_menu(request):

    tree_id = request.site.id
    root_node = Menu.tree.root_node(tree_id)
    if not root_node or not request.user.is_superuser:
        raise Http404

    if request.method == 'POST':
        try:
            # get tree from json-data
            tree = simplejson.loads(request.POST["mptt_tree"])

            # format tree, remove root-item and make root-parents = id
            root = None
            for node in tree:
                if node['parent_id'] == 'root':
                    node['parent_id'] = root_node.id
                if node['item_id'] == 'root':
                    root = node

            if root:
                tree.remove(root)

            cursor = connection.cursor()
            qn = connection.ops.quote_name
            opts = root_node._meta

            if settings.DATABASES['default']['ENGINE'].endswith("sqlite3"):
                cursor.executemany("""
                    UPDATE %(table)s
                    SET %(parent)s = :parent_id,
                        %(left)s = :left,
                        %(right)s = :right,
                        %(level)s = :depth
                    WHERE %(pk)s = :item_id AND %(tree_id)s = %(given_tree_id)s""" % {
                        "table": qn(opts.db_table),
                        'level': qn(opts.get_field(opts.level_attr).column),
                        'left': qn(opts.get_field(opts.left_attr).column),
                        'tree_id': qn(opts.get_field(opts.tree_id_attr).column),
                        'right': qn(opts.get_field(opts.right_attr).column),
                        'parent': qn(opts.get_field(opts.parent_attr).column),
                        'pk': qn(opts.pk.column),
                        'given_tree_id': tree_id,
                    }, tree)
            else:
                cursor.executemany("""
                    UPDATE %(table)s
                    SET %(parent)s = %%(parent_id)s,
                        %(left)s = %%(left)s,
                        %(right)s = %%(right)s,
                        %(level)s = %%(depth)s
                    WHERE %(pk)s = %%(item_id)s AND %(tree_id)s = %(given_tree_id)s""" % {
                        "table": qn(opts.db_table),
                        'level': qn(opts.get_field(opts.level_attr).column),
                        'left': qn(opts.get_field(opts.left_attr).column),
                        'tree_id': qn(opts.get_field(opts.tree_id_attr).column),
                        'right': qn(opts.get_field(opts.right_attr).column),
                        'parent': qn(opts.get_field(opts.parent_attr).column),
                        'pk': qn(opts.pk.column),
                        'given_tree_id': tree_id,
                    }, tree)

            transaction.commit_unless_managed()

        except (KeyError, ValueError):
            pass

        return HttpResponseRedirect(reverse("menu-list"))

    menu = root_node.get_descendants().all()

    return render_to_response('samklang_menu/list.html',
        {
            'root': root_node,
            'menu': menu,
        },
        context_instance=RequestContext(request))



def new_menuitem(request):
    """Create a new menu item - return html for ajax injection"""
    tree_id = request.site.id
    if request.method == 'POST' and request.user.is_superuser:
        form = MenuForm(request.POST)
        if form.is_valid():
            menu = form.save(commit=False)
            if not Menu.objects.filter(tree_id=tree_id, url=menu.url):
                menu.user = request.user
                menu.insert_at(menu.parent, 'last-child', True)
            #TODO: invalidate
        return HttpResponseRedirect(reverse("menu-list"))
    else:
        menu = Menu()
        menu.tree_id = tree_id
        if request.method == 'GET' and 'parent_id' in request.GET:
            menu.parent_id = request.GET['parent_id']
        form = MenuForm(instance=menu)

    return render_to_response(
        'samklang_menu/new_item.html',
        {'form': form},
        context_instance=RequestContext(request))

def delete_menuitem(request):
    """Delete a menu item"""
    tree_id = request.site.id
    if request.method == 'POST' and 'node_id' in request.POST and request.user.is_superuser:
        node = Menu.objects.get(pk=request.POST['node_id'], tree_id=tree_id)
        node.delete()
        return HttpResponse("1")

    return HttpResponse("0")



from django.contrib import admin
from mptt.admin import MPTTModelAdmin
from samklang_menu.models import Menu, Widget


class WidgetInline(admin.TabularInline):
    model = Widget


class MenuAdmin(MPTTModelAdmin):
    inlines = [ WidgetInline ]

admin.site.register(Menu, MenuAdmin)



from samklang_menu.models import Menu
from django import template
from django.core.exceptions import ObjectDoesNotExist
from django.contrib.sites.models import Site

register = template.Library()

def html_menu(menu, active_link=None):
    menu_contents = []
    for item in menu:
        if item==active_link:
            class_expr = 'class="active"'
        else:
            class_expr = ''
        menu_contents.append('<li><a href="%s" %s>%s</a></li>' % (item.url, class_expr, item.name))
    return "<ul>" + "".join(menu_contents) + "</ul>"


@register.simple_tag
def simple_menu(request, default_url="/"):

    if hasattr(request, 'site'):
        site = request.site
    else:
        site = Site.objects.get(pk=1)

    # find the closest parent to the active url
    active_link = Menu.find_active(site, request.path_info)

    # find the menu closest to the active_link
    if active_link:
        menu = active_link.submenu(request.user)
    else:
        menu = Menu.objects.get(tree_id=site.id, url="/").submenu(request.user)

    return html_menu(menu, active_link)

@register.simple_tag
def menu(request, default_url="/"):
    return simple_menu(request, default_url)


@register.simple_tag
def breadcrumbs(request, extra=[]):
    """Get the menu item for the page you are at. Find all ancestors in the menu hierarchy. Start at top level and break if permissions are not ok.

    You can pass extra elements using the "extra" parameter"""

    if hasattr(request, 'site'):
        site_id = request.site.id
    else:
        site_id = 1
    user = request.user
    path = request.path_info
    try:
        link = Menu.objects.get(tree_id=site_id, url=path)
    except Menu.DoesNotExist:
        link = Menu.objects.get(tree_id=site_id, url='/')

    ancestors = []
    for ancestor in link.get_ancestors():
        if not ancestor.group or ancestor.group in user.groups.all():
            ancestors.append(ancestor)
        else:
            break
    ancestors.append(link)

    return " > ".join([ '<a href="%s">%s</a>' % (ancestor.url, ancestor.name) for ancestor in ancestors ] + [ '<a href="%s">%s</a>' % (t[0], t[1]) for t in extra ])

@register.simple_tag
def widgets(request, name):
    if request.active:
        return "".join([ widget.widget().render(request) for widget in request.active.widgets.all() ])
    else:
        return ""



#!/usr/bin/env python
from distutils.core import setup

setup(
        name='samklang-menu',
        version="0.3.1",
        author='Sigurd Gartmann',
        author_email='sigurdga-samklang@sigurdga.no',
        url='http://github.com/sigurdga/samklang-menu',
        description='Horizontal tree menu with breadcrumbs for Samklang',
        long_description=open('README.txt').read(),
        license="AGPL",
        packages = ['samklang_menu', 'samklang_menu.templatetags', 'samklang_menu.migrations'],
        package_data = {'samklang_menu': ['templates/samklang_menu/*.html', 'static/img/*.png', 'static/js/*.js', 'locale/*/LC_MESSAGES/django.*o']},
        py_modules=['samklang_menu.widgets'],
        install_requires=['django-mptt>=0.5.1'],
        classifiers=[
                "Development Status :: 3 - Alpha",
                "License :: OSI Approved :: GNU Affero General Public License v3",
                "Intended Audience :: Developers",
                "Framework :: Django",
                "Environment :: Web Environment",
                "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
                ]
        )



class Widget(object):

    def __init__(self, options, *args, **kwargs):
        super(Widget, self).__init__(*args, **kwargs)
        self.options = options

    def get_display_name(self):
        raise NotImplementedError

    def render(self, request):
        raise NotImplementedError

    def render_option_form(self):
        raise NotImplementedError

    def get_option_dict(self):
        return self.options
